using System;
using System.Collections;
using System.Text;
using System.Xml;
using System.Windows.Forms;

using Berkeley.CS160.ExperienceLogger.Questions;

namespace Berkeley.CS160.ExperienceLogger
{
    public class Logger
    {
        private XmlDocument m_doc;
        private IDictionary m_questions = new Hashtable();
        private XmlNode m_questionsNode;
        private XmlNode m_sessionsNode;
        private XmlNode m_rootNode;

        public const int WIDTH = 240;
        public const int HEIGHT = 320;

        public Logger()
        {
            m_doc = new XmlDocument();
            XmlNode decl = m_doc.CreateXmlDeclaration("1.0", "", "yes");
            m_doc.AppendChild(decl);

            m_rootNode = m_doc.CreateNode(XmlNodeType.Element, "Logger", "");
            m_doc.AppendChild(m_rootNode);
          
            m_questionsNode = m_doc.CreateNode(XmlNodeType.Element, "Questions", "");
            m_rootNode.AppendChild(m_questionsNode);

            m_sessionsNode = m_doc.CreateNode(XmlNodeType.Element, "Sessions", "");
            m_rootNode.AppendChild(m_sessionsNode);
        }

        public IDictionary Questions
        {
            get { return m_questions; }
        }

        public Session CreateSession(string name)
        {
            XmlNode node = CreateSessionNode();
            XmlAttribute nameAtt = m_doc.CreateAttribute("name");
            nameAtt.Value = name;
            node.Attributes.Append(nameAtt);

            return new Session(node, this);
        }

        public Session CreateSession()
        {
            return new Session(CreateSessionNode(), this);
        }

        private XmlNode CreateSessionNode()
        {
            XmlNode node = m_doc.CreateNode(XmlNodeType.Element, "Session", "");
            m_sessionsNode.AppendChild(node);

            XmlAttribute idAtt = m_doc.CreateAttribute("id");
            idAtt.Value = OpenNETCF.GuidEx.NewGuid().ToString();
            node.Attributes.Append(idAtt);

            XmlAttribute startTime = m_doc.CreateAttribute("start_time");
            startTime.Value = DateTime.Now.ToShortDateString() +" " + DateTime.Now.ToLongTimeString();
            node.Attributes.Append(startTime);

            return node;
        }

        public void RegisterQuestion(AQuestion question) 
        {
            m_questions[question.Name] = question;
            XmlNode node = question.CreateDescription(this);
            m_questionsNode.AppendChild(node);
        }

        public void Save(string fileName)
        {
            XmlTextWriter output = new XmlTextWriter(fileName, Encoding.UTF8);
            m_doc.WriteTo(output);
            output.Close();
        }

        public XmlElement CreateElement(string name)
        {
            return m_doc.CreateElement(name);
        }

        public XmlAttribute CreateAttribute(string name)
        {
            return m_doc.CreateAttribute(name);
        }

        public static Form CreatePopupForm()
        {
            Form f = new Form();

            //f.AutoScaleDimensions = new System.Drawing.SizeF(96F, 96F);
            //f.AutoScaleMode = System.Windows.Forms.AutoScaleMode.Dpi;
            //f.AutoScroll = true;
            f.ClientSize = new System.Drawing.Size(240, 320);
            //f.KeyPreview = true;

            return f;
        }

        public static FlowLayout.FlowLayoutPanel CreateLayoutPanel()
        {
            FlowLayout.FlowLayoutPanel panel = new FlowLayout.FlowLayoutPanel();
            panel.LayoutStyle = FlowLayout.LayoutStyle.FlowLayout;
            panel.Width = Logger.WIDTH;
            panel.Height = Logger.HEIGHT;

            return panel;
        }
    }
}
