using System;
using System.Collections;
using System.Text;
using System.Xml;

namespace Berkeley.CS160.ExperienceLogger.Questions
{
    public abstract class AQuestion
    {
        private IList m_prompts = new ArrayList();
        private string m_name;

        public AQuestion(string name)
        {
            m_name = name;
        }

        public abstract AQuestionInstance CreateInstance();

        /// <summary>
        /// Create a description of the question to go in the Questions section
        /// of the log file.  The returned XmlNode must have a name attribute,
        /// a prompt_type attribute, and children for each prompt in the question.
        /// </summary>
        /// <seealso cref="CreateDefaultDescription"/>
        /// <param name="doc">The document that this node will go into</param>
        public abstract XmlNode CreateDescription(Logger logger);

        /// <summary>
        /// Creates the default description node and descriptions for all of the
        /// prompts.  The returned XmlNode must a prompt type set before it is
        /// returned by CreateDescription().
        /// </summary>
        protected XmlNode CreateDefaultDescription(Logger logger)
        {
            XmlNode node = logger.CreateElement("Question");
            XmlAttribute nameAtt = logger.CreateAttribute("name");
            nameAtt.Value = Name;
            node.Attributes.Append(nameAtt);

            foreach (IPrompt p in Prompts)
            {
                node.AppendChild(p.CreateDescription(logger));
            }

            return node;
        }

        public IList Prompts
        {
            get { return m_prompts; }
        }

        public string Name
        {
            get { return m_name; }
        }
    }
}
