using System;
using System.Collections;
using System.Text;
using System.Xml;
using Microsoft.WindowsMobile.Forms;

namespace Berkeley.CS160.ExperienceLogger.Questions
{
    public class ImageCaptureQuestion : AQuestion
    {
        private string m_title;
        private static int m_currentIndex = 0;

        public ImageCaptureQuestion(string name, string title) : base(name) 
        {
            m_title = title;
        }

        string GetNextFilename()
        {
            m_currentIndex++;
            return Name + "_" + m_currentIndex.ToString() + ".jpg";
        }

        public override System.Xml.XmlNode CreateDescription(Logger logger)
        {
            XmlNode node = this.CreateDefaultDescription(logger);
            XmlAttribute typeAtt = logger.CreateAttribute("node_type");
            typeAtt.Value = "image";
            node.Attributes.Append(typeAtt);

            XmlAttribute questAtt = logger.CreateAttribute("prompt");
            questAtt.Value = m_title;
            node.Attributes.Append(questAtt);

            return node;
        }

        public override AQuestionInstance CreateInstance()
        {
            return new ImageCaptureQuestionInstance(this);
        }

        private class ImageCaptureQuestionInstance : AQuestionInstance
        {
            CameraCaptureDialog m_dialog;
            ImageCaptureQuestion m_parent;

            public ImageCaptureQuestionInstance(ImageCaptureQuestion parent) : base(parent)
            {
                m_parent = parent;
                
                m_dialog = new CameraCaptureDialog();
                m_dialog.DefaultFileName = m_parent.GetNextFilename();
                m_dialog.Mode = CameraCaptureMode.Still;
                m_dialog.Title = m_parent.m_title;
            }

            public override System.Windows.Forms.DialogResult ShowControl()
            {
                try
                {
                    return m_dialog.ShowDialog();
                }
                catch (Exception)
                {
                    System.Windows.Forms.MessageBox.Show("Tried to ask an image question, but the camera is not supported. (This is probably because you are running in an emulator.)", "Camera missing...", System.Windows.Forms.MessageBoxButtons.OK, System.Windows.Forms.MessageBoxIcon.Exclamation, System.Windows.Forms.MessageBoxDefaultButton.Button1);
                    return System.Windows.Forms.DialogResult.Cancel;
                }
            }

            public override XmlNode GetResponse(Logger logger)
            {
                XmlNode node = logger.CreateElement("Question");

                XmlAttribute nameAtt = logger.CreateAttribute("name");
                nameAtt.Value = Parent.Name;
                node.Attributes.Append(nameAtt);

                XmlAttribute filenameAtt = logger.CreateAttribute("filename");
                filenameAtt.Value = m_dialog.FileName;
                node.Attributes.Append(filenameAtt);

                return node;
            }
        };
    }
}
